#!python3
from eyes_soatra.funcs.view_page import view_page
from eyes_soatra.funcs.time_app import time_app
from eyes_soatra.funcs.listener.watch_changes import watch_changes
from eyes_soatra.funcs.utils.string import remove_invisible